
; software for the Diva Automatiom SuprDupr & SuprChip V boards $ver 0.3

; this software is to provide EhBASIC and associated hardware drivers
; for the SuprDupr board

; This code is (c) Lee Davison 2002/3 leeedavison@lycos.co.uk

; I/O port		allocation		EhBASIC support
; ========		==========		===============

; PORT 0	; A[0..7]
; PORT 1	; A[8..15]
; PORT 2	; D[0..7]

; PORT 3.0	; L149 enable
; PORT 3.1	;
; PORT 3.2	; /ONW
; PORT 3.3	; /RESET OUT
; PORT 3.4	; 
; PORT 3.5	; /SYNC
; PORT 3.6	; /WR
; PORT 3.7	; /RD

; PORT 4.0	; upper LED, green anode
; PORT 4.1	; I2C CLK
; PORT 4.2	;					INT0
; PORT 4.3	;					INT1
; PORT 4.4	; RxD
; PORT 4.5	; TxD
; PORT 4.6	;
; PORT 4.7	; I2C DATA

; PORT 5.0	; lower LED, green anode
; PORT 5.1	;					INT2
; PORT 5.2	; upper LED, red anode		INT3
; PORT 5.3	; lower LED, red anode		INT4
; PORT 5.4	;
; PORT 5.5	;
; PORT 5.6	;					D2A1
; PORT 5.7	;					D2A2

; PORT 6.0	;					A2D(0)
; PORT 6.1	;					A2D(1)
; PORT 6.2	; LCD E				A2D(2)
; PORT 6.3	;					A2D(3)
; PORT 6.4	;					A2D(4)
; PORT 6.5	;					A2D(5)
; PORT 6.6	; LCD /WR				A2D(6)
; PORT 6.7	; LCD RS				A2D(7)

; PORT 7.0	;
; PORT 7.1	;
; PORT 7.2	;
; PORT 7.3	;
; PORT 7.4	;
; PORT 7.5	;
; PORT 7.6	; AT Keyboard CLK
; PORT 7.7	; AT Keyboard DATA

; PORT 8	; LCD D[0..7]

; I/O port and special register names - these derived from the 3608 users manual

Port0		= $00			; port data register
PDir0		= Port0+1		; port direction register
Port1		= $02			; port data register
PDir1		= Port1+1		; port direction register
Port2		= $04			; port data register
PDir2		= Port2+1		; port direction register
Port3		= $06			; port data register
PDir3		= Port3+1		; port direction register
Port4		= $08			; port data register
PDir4		= Port4+1		; port direction register
Port5		= $0A			; port data register
PDir5		= Port5+1		; port direction register
Port6		= $0C			; port data register
PDir6		= Port6+1		; port direction register
Port7		= $0E			; port data register
PDir7		= Port7+1		; port direction register
Port8		= $10			; port data register
PDir8		= Port8+1		; port direction register

Tx1B		= $18			; serial I/O 1 Tx buffer
Rx1B		= Tx1B		; serial I/O 1 Rx buffer
SIO1_s	= $19			; serial I/O 1 status
TBE		= 0			; Tx buffer empty flag - 1 = empty
RBF		= 1			; Rx buffer full flag - 1 = full
TSC		= 2			; Tx shift complete flag - 1 = complete
OE		= 3			; overrun error flag - 1 = overrun
PE		= 4			; parity error flag - 1 = parity
FE		= 5			; framing error flag - 1 = framing
SE		= 6			; summing error flag - 1 = summing (OE#PE#FE)
SIO1_c	= $1A			; serial I/O 1 control
UART_c	= $1B			; UART control
BRG		= $1C			; baud rate generator
SIO2_c	= $1D			; serial I/O 1 control

Tx2B		= $1F			; serial I/O 2 Tx buffer
Rx2B		= Tx2B		; serial I/O 2 Rx buffer
PRE12		= $20			; timer 1 & 2 prescaler
T1		= $21			; timer 1
T2		= $22			; timer 2
TM		= $23			; timer X & Y mode register
PREX		= $24			; timer X prescaler
TX		= $25			; timer X
PREY		= $26			; timer Y prescaler
TY		= $27			; timer Y

AD_c		= $34			; AD/DA control
AD		= $35			; A-D conversion
DA1		= $36			; D-A 1 conversion
DA2		= $37			; D-A 2 conversion

INT_e		= $3A			; interrupt edge selection
CPUM		= $3B			; CPU mode select
IREQ1		= $3C			; interrupt request register 1
IREQ2		= $3D			; interrupt request register 2
ICON1		= $3E			; interrupt control register 1
ICON2		= $3F			; interrupt control register 2

; $40 to $FF is RAM

LAB_0000	= $0040		; BASIC warm start

; monitor variables

A_reg		= $00E0		; break store, accumulator
X_reg		= $00E1		; break store, X register
Y_reg		= $00E2		; break store, Y register
P_reg		= $00E3		; break store, status register
K_reg		= $00E4		; break store, stack pointer
PCLreg	= $00E5		; break store, program counter low
PCHreg	= $00E6		; break store, program counter high
brkval	= $00E7		; store for opcode replaced by break

go_l		= PCLreg		; store for go addr in save, new block start
go_h		= PCHreg		; address in move & edit cursor location

to_l		= $00E8		; store for to addr in move save and display
to_h		= $00E9		; high byte of to address

store	 	= $00EB		; store for data in data mode and other subs
count		= $00EC		; count for tabular display
from_l	= $00ED		; store for current and from address lo
from_h	= $00EE		; store for current and from address hi

LAB_01FF	= $01FF		; stack top

ccflag	= $0200		; BASIC CTRL-C flag, 00 = en, 01 = dis
ccbyte	= ccflag+1		; BASIC CTRL-C byte
ccnull	= ccbyte+1		; BASIC CTRL-C byte timeout

VEC_CC	= ccnull+1		; ctrl c check vector
VEC_IN	= VEC_CC+2		; input vector
VEC_OUT	= VEC_IN+2		; output vector
VEC_LD	= VEC_OUT+2		; load vector
VEC_SV	= VEC_LD+2		; save vector

					; start of vector table
vec_brk	= VEC_SV+2		; software interrupt (BRK)
vec_a2d	= vec_brk+3		; A to D conversion finished
vec_int4	= vec_a2d+3		; INT4 condition met
vec_int3	= vec_int4+3	; INT3 condition met
vec_int2	= vec_int3+3	; INT2 condition met
vec_sio2	= vec_int2+3	; serial I/O 2 transfer complete
vec_cntr1	= vec_sio2+3	; CNTR1 condition met
vec_cntr0	= vec_cntr1+3	; CNTR0 condition met
vec_tmr2	= vec_cntr0+3	; timer 1 underflow
vec_tmr1	= vec_tmr2+3	; timer 2 underflow
vec_tmry	= vec_tmr1+3	; timer X underflow
vec_tmrx	= vec_tmry+3	; timer Y underflow
vec_siotx	= vec_tmrx+3	; serial I/O 1 Tx complete
vec_siorx	= vec_siotx+3	; serial I/O 1 Rx complete
vec_int1	= vec_siorx+3	; INT1 condition met
vec_int0	= vec_int1+3	; INT0 condition met
					; end of vector table
vec_end	= vec_int0+3	; next free byte

 .include	basic.asm		; EhBasic for the suprdupr


LANGROM	= $C000		; start of EhBASIC

	*=	$F800

; ***********************************************************************************

; reset code. initialisation starts here

int_res:
	SEI				; disable interrupts
	CLT				; clear T flag
	CLD				; clear decimal mode

	LDX	#$FF			; initial stack pointer
	TXS				; initialise stack

	LDM	#$B0,SIO1_c		; set serial I/O 1 control register
					; bit 7 = 1 = enable
					; bit 6 = 0 = UART mode
					; bit 5 = 1 = enable Rx
					; bit 4 = 1 = enable Tx
					; bit 3 = 0 = interrupt on Tx empty
					; bit 2 = 0 = P4.7 not /SRDY
					; bit 1 = 0 = use bit rate generator/16
					; bit 0 = 0 = bit rate generator use fin
	LDM	#0,UART_c		; set UART control register
					; bit 7,6,5 = unused
					; bit 4 = 0 = TxD is CMOS type output
					; bit 3 = 0 = 1 stop bit
					; bit 2 = 0 = even parity
					; bit 1 = 0 = no parity check
					; bit 0 = 0 = 8 bits

;;	LDM	#$33,BRG		; set baud rate (8MHz XTAL)
					; 
					; bit rate is fbcl/(n+1) wher fbcl is the effective
					; master bit clock which in this case is fin/16
					; 
					; so for 9600 baud	n = (fbcl/9600)-1
					;				n = ((8E6/16)/9600)-1
					;				n = (5E5/9600)-1
					;				n = 52-1 = 51 = $33
					;
					; this actually gives 9615 baud - close enough

	LDM	#$2F,BRG		; set baud rate (7.3728MHz XTAL)
;;					; 
;;					; bit rate is fbcl/(n+1) wher fbcl is the effective
;;					; master bit clock which in this case is fin/16
;;					; 
;;					; so for 9600 baud	n = (fbcl/9600)-1
;;					;				n = ((7.3728E6/16)/9600)-1
;;					;				n = (460800/9600)-1
;;					;				n = 48-1 = 47 = $2F
;;					;
;;					; this gives exactly 9600 baud

	LDM	#6,CPUM		; put the stack in page 1
;	SEB	2,CPUM		; put the stack in page 1

					; this needed to enable the MAX3311 on the SuprChip V
	SEB	0,PDir3		; set MAX chip enable bit to output
	SEB	0,Port3		; set MAX chip enable bit to 1

					; initialise page 2 variables
	LDY	#PG2_ED-PG2_TB	; byte count
PG2_LOOP:
	LDA	PG2_TB-1,Y		; get byte
	STA	VEC_IN-1,Y		; store in page 2
	DEY				; decrement count
	BNE	PG2_LOOP		; loop if not done

LAB_BTMSG:
	LDA	TAB_MSG,Y		; get byte from start-up message
	JSR	OUTPUT		; call output routine
	INY				; next byte
	CPY	#LAB_EMS-TAB_MSG	; compare with loop limit
	BNE	LAB_BTMSG		; loop if not complete

WAIT_LP:
	JSR	RINPUT		; call input routine
	BCC	WAIT_LP		; loop if no key

	AND	#$DF			; mask case bit (make upper case)
	CMP	#"W"			; compare with 'W' (warm)
	BNE	LAB_NWRM		; branch if not

	JMP	LAB_0000		; jump to warm start vector

; if you don't want the monitor the next three lines of code can be omitted

LAB_NWRM:
	CMP	#"M"			; compare with 'M' (monitor)
	BNE	LAB_NMON		; branch if not

	JMP	NewMon		; jump to monitor

LAB_NMON:
	CMP	#"C"			; compare with 'C' (cold)
	BNE	WAIT_LP		; loop if not

	JMP	LANGROM		; go do BASIC cold start


; if you don't want the monitor everything between the '####' lines can be omitted

; ################################################################################

; tabular display routine

LAB_F90C:
	JSR	TWOQAD		; get 'from' & 'to' addresses
LAB_F90F:
	JSR	CRLF			; output [CR][LF]
	JSR	QDDATA		; output 'from' address as hex
	JSR	SPCOUT		; output ASCII space
	JSR	LAB_FBE0		; output '>'
	LDX	#$10			; bytes per line count
	STX	count			; save it
LAB_F91F:
	JSR	SPCOUT		; output ASCII space
	JSR	PRBYTE		; outputs byte at ('from'),Y as hex
	JSR	NOTEND		; compare 'from' and 'to' addresses
	BCS	LAB_F97B		; if done set stack and return to monitor

	JSR	BUMP			; increment 'from' address
	DEC	count			; decrement byte count
	BEQ	LAB_F90F		; go do new header if 8 bytes done
	BNE	LAB_F91F		; go do next byte

; monitor move routine entry point. aaaa is the start of the block and bbbb
; is the /inclusive/ block end. cccc is the first address of the blocks
; destination.

DO_MOVE:
	JSR	TRIQAD		; get aaaa,bbbb>cccc
	JSR	SWAP			; move block 'from' to 'to' to 'go'
	BCS	MSTART		; jump, as carry always set, back to
					; command/address loop

; restart from breakpoint. gets restart address and register contents from
; the break table. then restarts by executing an RTI instruction.

DO_RTI:
	LDX	K_reg			; get back user stack
	TXS				; set stack register
	LDA	PCHreg		; get user PC high
	PHA				; push it
	LDA	PCLreg		; get user PC low
	PHA				; push it
	LDA	P_reg			; get processor status
	PHA				; push it
	LDA	A_reg			; restore accumulator
	LDX	X_reg			; restore X reg
	LDY	Y_reg			; restore Y reg
	RTI				; go back to user code

; set breakpoint. syntax is Zaaaa where a is the address at which the break
; point is to be inserted.

SETBRK:
	LDX	#$03			; 3 bytes to copy

SBKCLP:				; set break copy loop
	LDA	BRKVEC-1,X		; copies break handler vector
	STA	vec_brk-1,X		; over the preset break vector
	DEX				; decrement byte count
	BNE	SBKCLP		; loop if not done

	JSR	GETNEW		; get key and echo to display
	JSR	GETQDE		; get rest of word, put at 'from' address
	LDA	(from_l),Y		; get byte from intended breakpoint
	STA	brkval		; remember it
	TYA				; zero into A
	STA	(from_l),Y		; sets the breakpoint
	BEQ	MSTART		; back to command/addres loop

; save routine

LAB_F968:
	JMP	Intel_out		; jump to intel save

; load routine

LAB_F96B:
	JMP	Intl_in		; jump to intel load routine

; tabular display

LAB_F96F:
	BEQ	LAB_F90C		; branch to tabular display

; error return to monitor loop

LAB_F972:
	LDA	#$3F			; '?'
	JSR	OUTPUT		; call output routine
LAB_F97B:
	LDX	#$30			; load X
	TXS				; clear stack

; main monitor loop. Entry to command/address mode.

MSTART:
	JSR	CRLF			; output [CR][LF]
	JSR	LAB_FBE0		; output '>'
LAB_F988:
	JSR	GETNEW		; get key and echo to display

	CMP	#"H"			; compare with H (help)
	BEQ	DO_HELP		; jump to help routine

	CMP	#"?"			; compare with ? (help)
	BEQ	DO_HELP		; jump to help routine

	CMP	#$4D			; compare with M (memory block move)
	BEQ	DO_MOVE		; jump to move routine

	CMP	#$52			; compare with R (restart from break)
	BEQ	DO_RTI		; branch to restart

	CMP	#$5A			; compare with Z (set breakpoint)
	BEQ	SETBRK		; branch to set break

	CMP	#$53			; compare with S (save machine code)
	BEQ	LAB_F968		; branch to save

	CMP	#$4C			; compare with L (set load flag)
	BEQ	LAB_F96B		; branch to intel in routine

	BRA	LAB_F9D6		; branch to rest of key checks

; get 'from' and 'to' addresses

TWOQAD:
	JSR	GETNEW		; get key and echo to display
	JSR	GETQDE		; get rest of word, index 'from'
	JSR	LAB_FBE3		; output ','
	LDX	#$00			; index F9,FA

; get word, put at index

LAB_F9B1:
	JSR	GETNEW		; get key and echo to display
	.byte	$2C			; dummy BIT instruction, skips LDX

; get rest of word, put at 'from' address

GETQDE:
	LDX	#$05			; index to 'from' address
	JSR	LAB_F9C0		; get rest of byte
	JSR	GETNEW		; get key and echo to display
	.byte	$2C			; dummy BIT instruction, skip LDXs

GETPRC:
	LDX	#$03			; index to FC (only if called here!)
LAB_F9C0:
	JSR	LAB_F9C6		; rotate nibble into index
	JSR	GETNEW		; get key and echo to display
LAB_F9C6:
	CMP	#$2E			; compare with '.' (return to addr mode)
	BEQ	LAB_F988		; escape to address/command mode loop
	CMP	#$2F			; compare with '/' (jump to data mode loop)
	BEQ	DATA_LP		; jump to data mode loop
	JSR	ASCHEX		; convert ASCII in A to hex nibble
	BMI	LAB_F972		; branch if there was an error
	JMP	ROLSTR		; rotate nibble in A left into F9,X FA,X and return

; help routine

DO_HELP:
	JMP	helpme		; jump to help

; continue key checks

LAB_F9D6:
	CMP	#$54			; compare with 'T' (tabular display)
	BEQ	LAB_F96F		; branch to tabular display

	JSR	GETQDE		; get rest of word, put at 'from' address
LAB_F9DD:
	LDA	#$2F			; make A = '/'
	JSR	OUTPUT		; call output routine
	JSR	PRBYTE		; outputs byte at ('from'),Y as hex
	JSR	SPCOUT		; output ASCII space

DATA_LP:
	JSR	GETNEW		; get key and echo to display
	CMP	#$47			; compare with 'G'
	BNE	NOT_GO		; branch if not 'go'

	JMP	(from_l)		; go from address

NOT_GO:
	CMP	#$2C			; compare with ','
	BNE	NOT_STOR		; branch if not store

	JSR	BUMP			; increment 'from' address
	JMP	DATA_LP		; back to data mode loop

NOT_STOR:
	CMP	#$0A			; compare with [LF] (increment address)
	BEQ	LAB_FA16		; branch to display address (newline) routine

	CMP	#$0D			; compare with [CR] (increment address)
	BEQ	LAB_FA1B		; branch to display address routine

	CMP	#$5E			; compare with '^' (decrement address)
	BEQ	LAB_FA21		; branch to display previous address routine

	CMP	#$27			; compare with ''' (enter text entry mode)
	BEQ	LAB_FA3A		; branch to enter text loop

	JSR	GETPRC		; else get rest of byte, put at store
	LDA	store			; get the byte
	STA	(from_l),Y		; store at current address (Y = 0)
LAB_FA13:
	JMP	DATA_LP		; back to data mode loop

; output address/byte on new line

LAB_FA16:
	LDA	#$0D			; [CR]
	JSR	OUTPUT		; call output routine

; output address/byte on this line

LAB_FA1B:
	JSR	BUMP			; increment 'from' address
	JMP	LAB_FA31		; go output address/byte

; display previous address

LAB_FA21:
	SEC				; set carry for subtarct
	LDA	from_l		; get 'from' address low byte
	SBC	#$01			; subtract 1
	STA	from_l		; save it
	LDA	from_h		; get 'from' address high byte
	SBC	#$00			; subtract carry only
	STA	from_h		; save it

; entry to data mode loop. prints current address and it's contents.

DATALN:
	JSR	CRLF			; output [CR][LF]. now on new line

; output address/byte

LAB_FA31:
	JSR	QDDATA		; output word at 'from' address as hex
	JMP	LAB_F9DD		; go output /('from')

STO_CHR:
	JSR	LAB_FEF7		; store A @ ('from'),Y and inc 'from' addr

; enter text loop

LAB_FA3A:
	JSR	GETNEW		; get key and echo to display
	CMP	#$27			; compare with '''
	BNE	N_ETXT		; not end text so branch

	JSR	LAB_FBE3		; else output ','
	BNE	LAB_FA13		; back to data mode loop (branch always)

N_ETXT:
	CMP	#$0D			; compare with [CR]
	BEQ	DATALN		; entry to data mode loop

	BNE	STO_CHR		; go save the byte and continue loop

; end of monitor input handle loop ......

BRKVEC:
	JMP	MON_BRK		; jump to breakpoint handler. this is copied
					; to the break handler vector once a break
					; point is set.

; CEGMON's breakpoint handler. a set breakpoint will cause a jump, via the
; breakpoint vector, to here.

MON_BRK:
	STA	A_reg			; save the accumulator to the break table
	PLA				; get the processor status
	PHA				; push it back
	AND	#$10			; mask all but break flag ??
	BNE	WAS_BRK		; branch if it was a break
	LDA	A_reg			; restore the accumulator
	RTI				; exit

WAS_BRK:
	STX	X_reg			; save the X register to the break table
	STY	Y_reg			; save the Y register to the break table
	PLA				; get the processor status
	STA	P_reg			; save it to the break table
	CLD				; clear decimal mode
	SEC				; set carry for a subtract
	PLA				; get the return address low byte
	SBC	#$02			; subtract two ??
	STA	PCLreg		; save program counter low to break table
	PLA				; get the return address high byte
	SBC	#$00			; complete the 16 bit subtract
	STA	PCHreg		; save program counter high to break table
	TSX				; copy stack pointer to X
	STX	K_reg			; save stack pointer to break table
	LDY	#$00			; clear index
	LDA	brkval		; get opcode from break store
	STA	(PCLreg),Y		; restore opcode (removes breakpoint)
	LDA	#$E0			; start of break table
	STA	from_l		; set current addr low to break table start
	STY	from_h		; set current addr high to break table start
	BNE	DATALN		; entry to data mode loop (branch always)

; output '>'

LAB_FBE0:
	LDA	#$3E			; '>'
	.byte	$2C			; dummy BIT instruction, skips LDA

; output ','

LAB_FBE3:
	LDA	#$2C			; ','
	.byte	$2C			; dummy BIT instruction, skips LDA

; output ASCII space

SPCOUT:
	LDA	#$20			; ' '
	JMP	OUTPUT		; call output routine

; compare 'from' and 'to' addresses. exit with carry set if 'from' is
; greater than 'to'

NOTEND:
	SEC				; set for subtract
	LDA	from_l		; get low byte of 'from' address
	SBC	to_l			; subtract low byte of 'to' address
	LDA	from_h		; get high byte of 'from' address
	SBC	to_h			; subtract high byte of 'to' address
	RTS

; output [CR][LF]

CRLF:
	LDA	#$0D			; [CR]
	JSR	OUTPUT		; call output routine
	LDA	#$0A			; [LF]
	JMP	OUTPUT		; call output routine & return

; move block 'from' to 'to' to 'go' call from SWAP. Assumes Y = 00h

SWAP_LP:
	JSR	BUMP			; increment 'from' address
	INC	go_l			; next byte
	BNE	SWAP			; skip page incr if not needed
	INC	go_h			; next page

; routine entry point

SWAP:
	LDA	(from_l),Y		; Get byte from source
	STA	(go_l),Y		; copy to destination
	JSR	NOTEND		; compare 'from' and 'to' addresses
	BCC	SWAP_LP		; not done yet so loop
	RTS

; machine code monitor 'reset' entry point. resets vectors and pointers,
; clears decimal mode. recommended re-entry point for auto load m/c tapes.

NewMon:
	CLD				; clear decimal mode
	LDM	#$00,from_l		; zero from address low
	LDM	#$00,from_h		; zero from address high
	JMP	LAB_F97B		; set stack and go to command/address mode

; get key and echo to display

GETNEW:
	JSR	RINPUT		; get byte from ACIA
	BCC	GETNEW		; loop if no key

	AND	#$7F			; mask out top bit
	JMP	OUTPUT		; call output routine and return

; convert ASCII in A to hex nibble

ASCHEX:
	CMP	#$30			; compare with $30h
	BMI	LAB_FEA9		; branch if less
	CMP	#$3A			; compare with $3Ah
	BMI	LAB_FEA6		; branch if less (was 0-9)
	CMP	#$41			; compare with 41h
	BMI	LAB_FEA9		; branch if less (not hex key)
	CMP	#$47			; compare with 47h
	BPL	LAB_FEA9		; branch if greater or equal (not hex key)
	SEC				; set for subtract
	SBC	#$07			; adjust A-F
LAB_FEA6:
	AND	#$0F			; convert to hex nibble
	RTS

LAB_FEA9:
	LDA	#$80			; flag error
	RTS

; output word @ 'from' and byte @ store

ADVTOD:
	JSR	QDDATA		; output word at 'from' as hex
	JSR	SPCOUT		; output ASCII space
	BNE	PRDATA		; output byte in store as hex

; output word at 'from' as hex

QDDATA:
	LDX	#$03			; index high byte of 'from' address first
	JSR	LAB_FEBF		; outputs byte at store,X as hex
	DEX				; index low byte of 'from' address
	.byte	$2C			; dummy BIT instruction, skips LDX #00

; output byte in store as hex

PRDATA:
	LDX	#$00			; index store for output

; outputs byte at store,X as hex

LAB_FEBF:
	LDA	store,X		; get byte
	LSR	A			; rotate high nibble
	LSR	A			; into low nibble
	LSR	A			; 
	LSR	A			; 
	JSR	HEXOUT		; output low nibble of A as hex chr
	LDA	store,X		; get byte back

; output low nibble of A as hex chr

HEXOUT:
	AND	#$0F			; make sure in range 0-F
	ORA	#$30			; convert to ASCII
	CMP	#$3A			; check numeric
	BMI	LAB_FED5		; jump if so

	CLC				; clear carry
	ADC	#$07			; adjust A-F
LAB_FED5:
	JMP	OUTPUT		; call output routine and return

; rotate nibble in A left into to_l,X to_h,X

ROLSTR:
	LDY	#$04			; bit count is 4
	ASL	A			; move nibble to top nibble
	ASL	A			;
	ASL	A			;
	ASL	A			;
LAB_FEE0:
	ROL	A			; rotate bit into carry
	ROL	to_l,X		; rotate carry into to_l,X
	ROL	to_h,X		; rotate carry into to_h,X
	DEY				; decrement bit count
	BNE	LAB_FEE0		; loop if not done
	RTS

; outputs byte at ('from'),Y as hex

PRBYTE:
	LDA	(from_l),Y		; get byte
	STA	store			; save at store
	JMP	PRDATA		; output byte in store as hex

; store A @ ('from'),Y and inc 'from' address

LAB_FEF7:
	STA	(from_l),Y		; save byte

; increment 'from' address

BUMP:
	INC	from_l		; increment 'from' address low byte
	BNE	LAB_FEFF		; skip page increment if not needed

	INC	from_h		; increment 'from' address page
LAB_FEFF:
	RTS

; get aaaa,bbbb>cccc

TRIQAD:
	JSR	TWOQAD		; get 'from' and 'to' addresses
	JSR	LAB_FBE0		; output '>'
	LDX	#$03			; index FC,FD
	JSR	LAB_F9B1		; get word, put at index
	LDA	$FC			; last word low byte
	LDX	$FD			; last word high byte
	STA	go_l			; copy to E4
	STX	go_h			; copy to E5
	RTS

; output '.' then word at 'from' address as hex

LAB_FFE3:
	LDA	#$2E			; '.'
	JSR	OUTPUT		; call output routine
	JMP	QDDATA		; output word at 'from' address as hex
					; and then return.

; get intel hex record. short version - no checksum calculation.

Intl_in:
	LDY	#$00			; no index offset
Byte_lp:
	JSR	RINPUT		; get chr
	BCC	Byte_lp		; loop if no character

	CMP	#":"			; is it ":"?
	BEQ	Rec_st		; if so then get record

	CMP	#"."			; is it "."?
	BNE	Byte_lp		; if not then loop

	JMP	NewMon		; else re enter monitor

Rec_st:
	JSR	Byte_rd		; get record byte count
	TAX				; put byte count in X
	JSR	Byte_rd		; get record address hi byte
	STA	from_h		; put in pointer high byte
	JSR	Byte_rd		; get record address lo byte
	STA	from_l		; put in pointer low byte
	JSR	Byte_rd		; ignore record type
Data_in:
	JSR	Byte_rd		; get record data byte
	STA	(from_l),Y		; save byte to memory
	INY				; increment index
	DEX				; decrement byte count
	BNE	Data_in		; branch if not finished

	BEQ	Intl_in		; ignore checksum byte and [CR][LF]

; get 2 ASCII chrs as hex byte

Byte_rd:
	LDA	#$00			; clear A
	STA	store			; clear temp byte store
	JSR	Hex_con		; get byte and convert to hex
	ASL	A			; first nibble so
	ASL	A			; multiply by 16
	ASL	A			;
	ASL	A			;
	STA	store			; save hi nibble in temp
Hex_con:
	JSR	RINPUT		; get next chr
	BCC	Hex_con		; loop if no character

;	SEC
	SBC	#$30			; makes '0'-'9' equal 0-9
	CMP	#$0A			; is it 0-9 ?
	BMI	N_alpha		; if so miss next bit

	SBC	#$07			; else convert alpha (carry will be set)
N_alpha:
	ORA	store			; add hi nibble back
	RTS

; intel hex out routine entry point. aaaa is the start of the block
; and bbbb is the /inclusive/ block end

Intel_out:
	JSR	TWOQAD		; get 'from' & 'to' addresses
out_l_loop:
	JSR	CRLF			; output [CR][LF]
	SEC				; set carry for subtract
	LDA	to_l			; get to address low byte
	SBC	from_l		; subtract from address low byte
	STA	go_l			; save diff low byte
	LDA	to_h			; get to address high byte
	SBC	from_h		; subtract from address high byte
	STA	go_h			; save diff high byte
	BCC	out_exit		; exit if to < from
	BNE	line_10		; branch if high byte <> 0

	LDA	go_l			; else get low byte back

	CMP	#$0F			; else compare with $0F
	BCC	less_10

line_10:
	LDA	#$0F			; make this line $10-1 bytes
less_10:
	SEC				; set carry
	ADC	#$00			; increment count
	STA	$FD			; save count
	STA	store			; save count as byte for output
	CLC				; clear carry for add
	ADC	from_l		; add from address low byte
	CLC				; clear carry for add
	ADC	from_h		; add from address high byte
	STA	go_h			; save checksum so far
	LDA	#":"			; colon for start of line
	JSR	OUTPUT		; output ":" for line start
	JSR	PRDATA		; output byte count from store as hex
	JSR	QDDATA		; output 'from' address as hex
	LDA	#$00			; set record type
	STA	store			; save count as byte for output
	JSR	PRDATA		; output byte from store as hex

byte_loop:
	LDA	(from_l),Y		; get byte
	STA	store			; save at store
	CLC				; clear carry for add
	ADC	go_h			; add checksum byte
	STA	go_h			; save checksum byte
	JSR	PRDATA		; output byte in store as hex

	JSR	BUMP			; increment 'from' address
	DEC	$FD			; decrement byte count
	BNE	byte_loop		; loop if bytes(s) to do

	LDA	go_h			; get checksum
	STA	store			; save as byte to output
	JSR	PRDATA		; output byte in store as hex
	JMP	out_l_loop		; go do next line

out_exit:
	JMP	LAB_F97B		; set stack and return to monitor

helpme:
	LDY	#$00			; clear index
help_lp:
	LDA	help_msg,Y		; get byte from start-up message
	BEQ	help_ex		; exit if end marker

	JSR	OUTPUT		; call output routine
	INY				; next byte
	BRA	help_lp		; loop if not complete

help_ex:
	JMP	LAB_F97B		; set stack and return to monitor

help_msg:
	.byte	$0D,$0A
	.byte	"[H]|[?] - Help",$0D,$0A
	.byte	"[M]ssss,eeee>dddd",$0D,$0A
	.byte	"[R] - Restart",$0D,$0A
	.byte	"[Z] - breakpoint",$0D,$0A
	.byte	"[S]ssss,eeee",$0D,$0A
	.byte	"[L] - Load",$0D,$0A
	.byte	"[T]ssss,eeee",$0D,$0A
	.byte	"[,] - Data mode",$0D,$0A
	.byte	$0D,$0A
	.byte	"[G] - Go",$0D,$0A
	.byte	"[CR]|[LF] - Increment",$0D,$0A
	.byte	"[^] - Decrement",$0D,$0A
	.byte	"[']abcde'",$0D,$0A
	.byte	"[/] - Address",$0D,$0A,$00

; ################################################################################

; if you don't want the monitor use the second prompt string

; start-up message

TAB_MSG:
	.byte	$0D,$0A,"SuprDupr & SuprChip V Mon Ver 0.3 (C) 2002/3"
	.byte	$0D,$0A,"Enhanced 740 BASIC [C]old/[W]arm/[M]onitor ?"

;	.byte	$0D,$0A,"SuprDupr & SuprChip V Mon Ver 0.3 (C) 2002/3"
;	.byte	$0D,$0A,"Enhanced 740 BASIC [C]old/[W]arm ?"
LAB_EMS:

; table of initial values for page 2

PG2_TB:
	.word	RINPUT		; input vector
	.word	OUTPUT		; output vector
	.word	SETLOD		; load vector
	.word	SETSAV		; save vector

; these are the interrupt vector destinations for all the 3806 interrupts.
; changing the destination can be done by the user by changing the word in RAM.

	JMP	int_brk		; software interrupt, BRK (non maskable)
	JMP	int_a2d		; A to D conversion finished
	JMP	int_4			; INT4 condition met
	JMP	int_3			; INT3 condition met
	JMP	int_2			; INT2 condition met
	JMP	int_sio2		; serial I/O 2 transfer complete
	JMP	int_cntr1		; CNTR1 condition met
	JMP	int_cntr0		; CNTR0 condition met
	JMP	int_tmr2		; timer 2 underflow
	JMP	int_tmr1		; timer 1 underflow
	JMP	int_tmrY		; timer Y underflow
	JMP	int_tmrX		; timer X underflow
	JMP	int_sioTx		; serial I/O 1 Tx complete
	JMP	int_sioRx		; serial I/O 1 Rx complete
	JMP	int_1			; INT1 condition met
	JMP	int_0			; INT0 condition met
PG2_ED:

; these are the support routines required by EhBASIC

; scan the input device
; if byte available return carry set and byte in A
; else return carry clear and $00 in A

RINPUT:
	LDA	SIO1_s		; get serial I/O 1 status
	AND	#$02			; mask rx buffer full flag
	CMP	#$02			; is bit set
	BNE	RD_EXIT		; exit with Cb=0 if buffer empty

	LDA	Rx1B			; get byte from ACIA data port (Cb=1)
RD_EXIT:
	RTS

; wait for serial I/O 1 and tx byte
; return sent byte in A and N and Z flags set

OUTPUT:
	BBC	TBE,SIO1_s,OUTPUT	; loop if tx buffer full

	STA	Tx1B		; save to data port
	RTS

; load BASIC program routine, default (do nothing)

SETLOD:

; save BASIC program routine, default (do nothing)

SETSAV:
	RTS

; interrupt routines. routines that are just RTI are not (yet) used by EhBASIC

int_brk:				; software interrupt, BRK (non maskable)
	RTI

int_a2d:				; A to D conversion finished
	RTI

; INT4 interrupt routine.

int_4:				; INT4 condition met
	PHA				; save A
	LDA	#$10			; set for INT4
	BRA	int_x			; go set bit

; INT3 interrupt routine.

int_3:				; INT3 condition met
	PHA				; save A
	LDA	#$08			; set for INT3
	BRA	int_x			; go set bit

; INT2 interrupt routine.

int_2:				; INT2 condition met
	PHA				; save A
	LDA	#$04			; set for INT2
	BRA	int_x			; go set bit

int_sio2:				; serial I/O 2 transfer complete
	RTI

; counter 1 interrupt routine. checks if in PWM mode and if so reads
; and resets timer Y prescaller and divider.

int_cntr1:				; CNTR1 condition met
	PHA				; save A
	BBC	4,TM,int_ctr1	; branch if not PWM mode
	BBC	5,TM,int_ctr1	; branch if not PWM mode

	LDA	PREY			; get prescaller Y byte
	STA	PwmYl			; save PWM mode timer Y word low byte
	LDA	TY			; get timer Y byte
	STA	PwmYh			; save PWM mode timer Y word high byte
	LDM	#$FF,PREY		; set prescaller Y to -1
	LDM	#$FF,TY		; set timer Y to -1
int_ctr1:
	LDA	#$02			; set for CNTR1
	BRA	int_ctrx		; go do rest of CNTRx interrupt

; counter 0 interrupt routine. checks if in PWM mode and if so reads
; and resets timer X prescaller and divider.

int_cntr0:				; CNTR0 condition met
	PHA				; save A
	BBC	0,TM,int_ctr0	; branch if not PWM mode
	BBC	1,TM,int_ctr0	; branch if not PWM mode

	LDA	PREX			; get prescaller X byte
	STA	PwmXl			; save PWM mode timer X word low byte
	LDA	TX			; get timer X byte
	STA	PwmXh			; save PWM mode timer X word high byte
	LDM	#$FF,PREX		; set prescaller X to -1
	LDM	#$FF,TX		; set timer X to -1
int_ctr0:
	LDA	#$01			; set for CNTR0
int_ctrx:
	AND	TmrBase+1		; AND with setup byte (mask if not setup)
	ORA	TmrBase+2		; OR with happened byte (set happened bit)
	STA	TmrBase+2		; save new happened byte
	PLA				; restore A
	RTI

; timer 2 underflow interrupt routine.

int_tmr2:				; timer 2 underflow
	PHA				; save A
	LDA	#$08			; set for timer X
	BRA	int_tmrQ		; go set bit

; timer 1 underflow interrupt routine.

int_tmr1:				; timer 1 underflow
	PHA				; save A
	LDA	#$04			; set for timer X
	BRA	int_tmrQ		; go set bit

; timer Y underflow interrupt routine.

int_tmrY:				; timer Y underflow
	PHA				; save A
	LDA	#$20			; set for timer X
	BRA	int_tmrQ		; go set bit

; timer X underflow interrupt routine.

int_tmrX:
	PHA				; save A
	LDA	#$10			; set for timer X
int_tmrQ:
	AND	TmrBase+1		; AND with setup byte (mask if not setup)
	ORA	TmrBase+2		; OR with happened byte (set happened bit)
	STA	TmrBase+2		; save new happened byte
	PLA				; restore A
	RTI

int_sioTx:				; serial I/O 1 Tx complete
	RTI

int_sioRx:				; serial I/O 1 Rx complete
	RTI

; INT1 interrupt routine.

int_1:				; INT1 condition met
	PHA				; save A
	LDA	#$02			; set for INT1
	BRA	int_x			; go set bit

; INT0 interrupt routine.

int_0:				; INT0 condition met
	PHA				; save A
	LDA	#$01			; set for INT0
int_x:
	AND	IntBase+1		; AND with setup byte (mask if not setup)
	ORA	IntBase+2		; OR with happened byte (set happened bit)
	STA	IntBase+2		; save new happened byte
	PLA				; restore A
	RTI

; 3806 vector table, all the vectors, except reset, point to jump instructions in RAM
; that can be changed by the user

	*=	$FFDC

	.word	vec_brk		; software interrupt, BRK (non maskable)
	.word	vec_a2d		; A to D conversion finished
	.word	vec_int4		; INT4 condition met
	.word	vec_int3		; INT3 condition met
	.word	vec_int2		; INT2 condition met
	.word	vec_sio2		; serial I/O 2 transfer complete
	.word	vec_cntr1		; CNTR1 condition met
	.word	vec_cntr0		; CNTR0 condition met
	.word	vec_tmr2		; timer 2 underflow
	.word	vec_tmr1		; timer 1 underflow
	.word	vec_tmry		; timer Y underflow
	.word	vec_tmrx		; timer X underflow
	.word	vec_siotx		; serial I/O 1 Tx complete
	.word	vec_siorx		; serial I/O 1 Rx complete
	.word	vec_int1		; INT1 condition met
	.word	vec_int0		; INT0 condition met

	.word	int_res		; reset

	.byte	$00,$02		; revision number (no NMI on 3806)

	.END



